@@ -1,6 +1,6 @@
 pickCoef <- function(object, regexpr = NULL, ...){
-    object <- names(coef(object))
-    if (is.null(object))
+    coefs <- names(coef(object))
+    if (is.null(coefs))
         stop("Coefficient names cannot be extracted from 'object'")
     if (missing(regexpr)) {
         default <- list(setlabels = "Selected coefficients",
@@ -10,15 +10,15 @@ pickCoef <- function(object, regexpr = NULL, ...){
                         warningText =  "No subset of coefficients selected")
         dots <- list(...)
         dotArgs <- match(names(default), names(dots))
-        allArgs <- c(list(object), dots, default[is.na(dotArgs)])
+        allArgs <- c(list(coefs), dots, default[is.na(dotArgs)])
         selection <- unname(unlist(do.call(relimp::pickFrom, allArgs)))
     }
     else
-        selection <- grep(regexpr, object)
+        selection <- grep(regexpr, coefs)
 
     if (!length(selection))
         selection <- NULL
     else
-        names(selection) <- object[selection]
+        names(selection) <- coefs[selection]
     selection
 }
