@@ -1,12 +1,12 @@
 # this should always be a summary based on single grouping factor
 summary.meanResiduals <- function (object, digits = max(3, getOption("digits") - 3), ...) {
-    cat("\nModel call:\n", deparse(object$call, width = options()$width),
+    cat("\nModel call:\n", deparse(object$call, width.cutoff = options()$width),
         "\n", sep = "", fill = TRUE)
 
     cat("Mean residuals by ", object$by,  ":\n\n", sep = "")
     q <- quantile(object$residuals, na.rm = TRUE)
     names(q) <- c("Min", "1Q", "Median", "3Q", "Max")
-    print.default(q, digits = digits, na = "", print.gap = 2)
+    print.default(q, digits = digits, na.print = "", print.gap = 2)
 
     if (object$standardized) {
         cat("\nTest of Normality:\n")
